import React from "react";
import { ReactNode } from "react";
import { Graph } from "../structures/graph/graph";
import { NodeType } from "../structures/graph/nodetype";
import "./nodetypeentry.css";

type propTypes = {
    graph: Graph;
    type: NodeType;
    onChange: { (): void };
};
type stateTypes = {
    temporaryColor: string;
};

export class NodeTypeEntry extends React.Component<propTypes, stateTypes> {
    constructor(props: propTypes) {
        super(props);
        this.deleteType = this.deleteType.bind(this);
        this.handleTextChange = this.handleTextChange.bind(this);

        this.setState({
            temporaryColor: undefined,
        });
    }

    private deleteType() {
        this.props.type.delete();
    }

    private isValidColor(color: string): boolean {
        if (color.length <= 0 || color[0] !== "#") {
            return false;
        }

        const colorCode = color.substring(1);

        if (!(colorCode.length === 3 || colorCode.length === 6)) {
            return false;
        }

        colorCode;
    }

    /**
     * Generic function for handeling a changing text input and applying the new value to the node type.
     * @param event Change event of text input.
     * @param property Property to give new value.
     */
    private handleTextChange(event: any, property: string) {
        const newValue = event.target.value;

        // Actual change?
        if ((this.props.type as any)[property] == newValue) {
            return;
        }

        //TODO: Make sure, that this event is not triggered to quickly!
        (this.props.type as any)[property] = newValue;
        this.props.type.graph.storeCurrentData(
            "Changed " + property + " of type [" + this.props.type + "]"
        );

        this.props.onChange();
    }

    render(): ReactNode {
        return (
            <li className="node-type">
                <input
                    className="node-type-name"
                    type={"text"}
                    value={this.props.type.name}
                    onChange={(event) => this.handleTextChange(event, "name")}
                />
                <input
                    className="node-type-color"
                    type={"text"}
                    value={
                        this.state.temporaryColor
                            ? this.state.temporaryColor
                            : this.props.type.color
                    }
                    onChange={(event) => this.handleTextChange(event, "color")}
                />
                {this.props.graph && this.props.graph.types.length > 1 ? (
                    <button onClick={this.deleteType}>Delete</button>
                ) : (
                    ""
                )}
            </li>
        );
    }
}
